﻿using FightLandloard.Modal;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FightLandloard.Modal
{
    public class CardGroupFactory
    {

        private CardGroupFactory()
        {

        }

        private static CardGroupFactory cfInstance = null;

        public static CardGroupFactory CreateInstance()
        {
            if(cfInstance == null)
            {
                cfInstance = new CardGroupFactory();
            }
            return cfInstance;
        }


        static CardGroupFactory()
        {
            LstAllCardTypes = new List<Type>()
            {
                typeof(BaseCardGroup),
                typeof(DoubleCardsGroup),
                typeof(DoubleSeqGroup),
                typeof(ThreeCardGroup),
                typeof(ThreeWithOneGroup),
                typeof(ThreeWithTowGroup),
                typeof(PlaneWithNothingGroup),
                typeof(PlaneWithSingleGroup),
                typeof(PlaneWithDoubleGroup),
                typeof(SingleSeqGroup),
                typeof(BomGroup),
                typeof(FourWithSingleGroup),
                typeof(FourWithDoubleGroup),
                typeof(BomPlaneWithNothingGroup),
                typeof(BomPlaneWithSingleGroup),
                typeof(BomPlaneWithDoubleGroup)

            };

            LstCardTypeWhenNumEqu5 = new List<Type>()
            {
                 typeof(SingleSeqGroup),
                  typeof(ThreeWithTowGroup)
            };
            LstCardTypeWhenNumEqu6 = new List<Type>()
            {
                 typeof(SingleSeqGroup),
                  typeof(DoubleSeqGroup),
                  typeof(FourWithSingleGroup),
                  typeof(PlaneWithNothingGroup)
            };
        }
        private static List<Type> LstAllCardTypes;
        private static List<Type> LstCardTypeWhenNumEqu5;
        private static List<Type> LstCardTypeWhenNumEqu6;
        public BaseCardGroup CreateByList(List<Card> list)
        {
            list.Sort();
            if (null == list || list.Count == 0)
            {
                return null;
            }

            var count = list.Count;

            if (count > 0 && count < 8)
            {

                if (count == 1)
                {

                    return new BaseCardGroup(list);

                }
                else if (count == 2)
                {
                    if (list[0].CardType != 5)
                    {
                        return new DoubleCardsGroup(list);
                    }
                    else
                    {
                        return new BomGroup(list);
                    }
                }
                else if (count == 3)
                {
                    return new ThreeCardGroup(list);
                }
                else if (count == 4)
                {
                    if (list[0].CardNum != list[3].CardNum)
                    {
                        return new ThreeWithOneGroup(list);
                    }
                    else
                    {
                        return new BomGroup(list);
                    }
                }
                else if (count == 5)
                {
                    return CreateByType(LstCardTypeWhenNumEqu5, list);

                }
                else if (count == 6)
                {
                    return CreateByType(LstCardTypeWhenNumEqu6, list);
                }
                else
                {
                    return new SingleSeqGroup(list);
                }
            }
            else
            {
                return CreateByType(LstAllCardTypes, list);
            }
        }

        private BaseCardGroup CreateByType(List<Type>listTyps, List<Card> list)
        {
            foreach (var item in listTyps)
            {
                var obj = CreateByType(item, list);

                if (obj.IsValid())
                {
                    return obj;
                }

            }
            return null;
        }

        private BaseCardGroup CreateByType(Type t, List<Card> list)
        {
            return (BaseCardGroup)Activator.CreateInstance(t, list);
        }

        public BaseCardGroup CreateByCardType(Type t, List<Card> list)
        {
            if (LstAllCardTypes.Contains(t))
            {
                return CreateByType(t, list);
            }
            else
            {


                return null;
            }

        }

    }
}
